package biology.spectro.collaboration;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Scanner;
/**
 * FileMgmt is the base class for managing files(.txt) operation including 
 * reading data from a file and writing data into a file.
 * @author    Chao Han
 * @version   1.0  Nov 11 2012
 * @since     1.7
 */
public class FileMgmt {
	/**
	 * The content of the file.
	 * @see    java.util.Scanner
	 */
	protected Scanner fileData;

	/**
	 * This class is used to get the content of the file
	 * @return the content of the file
	 */
	public Scanner getData(){
		return fileData;
	}
	
	/**
	 * Open a specified file
	 * @param       fileName the file to be open
	 */
	public  void openFile(String fileName){
		try{
			fileData = new Scanner(new File(fileName));
		}
		catch(Exception e){
			System.out.println("file is not found");   
		}
	}
	
	/**
	 * Close a specified file
	 * @param    fileName the file to be closed
	 */
	public void closeFile(String fileName){
		fileData.close();
	}
	
	/**
	 * Read a file, and Return the content of that file in String
	 * @param    fileName the file name of a txt file that store data
	 * @return   the content of the file in String 
	 */
	public String readFile(String fileName){
		openFile(fileName);
		String FileContent = "";
		while(fileData.hasNextLine()){
                    FileContent += fileData.nextLine();
                    FileContent += "\n";
                }
		closeFile(fileName);
		return FileContent;
	}
	
	/**
	 * Returns the content of a line in file at the specified index.
	 * @param     fileName the name of a file read from
	 * @param     lineIndex the index of the desired line
	 * @return    the content of the desired line
	 */
	public String readLine(String fileName, int lineIndex){
		openFile(fileName);
		java.util.List <String> LineContent = new ArrayList<String>();
		while(fileData.hasNext()){
			LineContent.add(fileData.nextLine());
		}
		return LineContent.get(lineIndex);
	}
	
	/**
	 * Write specified data to a specified file in terms of appending
	 * data to the end of the file or not.
	 * @see         java.io.FileWriter
	 * @param       fileName name of the file to be written
	 * @param       fileContent the content to be written
	 * @param       appendOrNot append to the original file data or not
	 * @exception   IOEception
	 *                if the file cannot be found
	 */
	public void writeFile(String fileName, String fileContent, boolean appendOrNot ){
		try {
			FileWriter  fileWr = new FileWriter(fileName,appendOrNot);
			fileWr.write(fileContent);
			fileWr.close();
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}