package biology.spectro.collaboration;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.BorderFactory;
/**
 * This class is used to create GUI for Login. This class extends JFrame,
 * which contains all account information and two buttons. 
 * @see     JFrame
 * @author  Chao Han
 *
 */
public class LoginGUI extends JFrame{
	/**
	 * Components on the GUI
	 */
	private JLabel          idLabel;
	private JLabel          pwLabel;
	
	private JTextField      idField;                           
	private JPasswordField  pwField;                        
	private JRadioButton    teacherButton;                    
	private JRadioButton    studentButton;
	private ButtonGroup     acountTypeButtonGroup;
	private JButton         signInButton;
	private JButton         exitButton;
	
	/**
	 * This is the constructor of LoginGUI, which create all components 
	 * on GUI. 
	 * <p>
	 * An account in this application contains three parts, ID, password 
	 * and account type. Therefore, this GUI should be composed of one 
	 * field for inputing id, one field to input password, two JRadioButton 
	 * to select account type, and two buttons, one is for sign in and the
	 * other is for exit. 
	 * @see     Account
	 */
	public LoginGUI(){
		/*title show at the top of the window*/
		super("Spectrophotometer");
		setLayout(new BorderLayout());
		
		idLabel = new JLabel("enter ID");
		pwLabel = new JLabel("enter PASSWORD");

		/*field for inputing id and field for inputing password*/
		idField = new JTextField(15);
		pwField = new JPasswordField(15);
		
		/*buttons for choosing account type*/
		teacherButton = new JRadioButton("teacher",true);
		studentButton = new JRadioButton("student",false);
		acountTypeButtonGroup = new ButtonGroup();
		acountTypeButtonGroup.add(teacherButton);
		acountTypeButtonGroup.add(studentButton);
		
		/*sign in button and exit button*/
		signInButton = new JButton("Sign In");
		exitButton = new JButton("  Exit  ");
		
		/*add listener to sign in and exit buttons*/
		SignInHandler signInHandler = new SignInHandler();
		signInButton.addActionListener(signInHandler);
		ExitHandler exitHandler = new ExitHandler();
		exitButton.addActionListener(exitHandler);
		
		/*
		 * after inputing password, press enter on keyboard equals to click
		 * Sign In button
		 */
		pwField.addActionListener(signInHandler);
		
		/*interface design and components placement*/
		JPanel LabelPanel = new JPanel(new GridLayout(0,1));
		LabelPanel.add(idLabel);
		LabelPanel.add(pwLabel);
		
		JPanel FieldPanel = new JPanel(new GridLayout(0,1));
		FieldPanel.add(idField);
		FieldPanel.add(pwField);
		FieldPanel.add(studentButton);
		LabelPanel.add(teacherButton);
		
		JPanel ButtonPanel = new JPanel(new FlowLayout());
		ButtonPanel.add(signInButton);
		ButtonPanel.add(exitButton);
		
		Border paneEdge = BorderFactory.createEmptyBorder(20,20,20,20);
		rootPane.setBorder(paneEdge);
		add(LabelPanel,BorderLayout.CENTER);
		add(FieldPanel,BorderLayout.LINE_END);
		add(ButtonPanel,BorderLayout.SOUTH);
	}
	
	/**
	 * Returns int which presents the selected account type
	 * @return   selected account type
	 *           1 is teacher, 2 is student
	 */
	public int getAccountType(){
		if(teacherButton.isSelected()== true)
			return 1;
		if(studentButton.isSelected()== true)
			return 2;
		return 0;
	}
	
	/**
	 * Get All input account information
	 * @see       Account
	 * @return    input account
	 */
	public Account readAccountInfo(){
		Account a= new Account(idField.getText(),pwField.getPassword(),getAccountType());
		a.printAccount();
		return  a;
	}
	
	/**
	 * Create and show GUI
	 */
	public static void createAndShwoGUI(){

        LoginGUI logIn = new LoginGUI();
        logIn.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        //Display the window.
        logIn.pack();
        logIn.setVisible(true);
	}

	/**
	 * This is a private class that implements interface ActionListener. When the Sign In 
	 * button is clicked, the account information input compare with accounts stored in 
	 * data file. If the input account can be found in the data file, enter to another GUI
	 * depending on account type. Otherwise, display the input account invalid. 
	 * @see    ActionListener
	 * @author Chao Han
	 *
	 */
	private class SignInHandler implements ActionListener{
		@Override
		public void actionPerformed(ActionEvent event){
			AccountStorage storageFile = new AccountStorage();
			
			int NumsOfAccount = storageFile.calulateAmountOfAccount("src/biology/spectro/resources/AccountStorage.txt");   
			
			storageFile.openFile("src/biology/spectro/resources/AccountStorage.txt");
			
			while(NumsOfAccount!=0){
				
				/*if an account is found, break*/
				if(AccountChecker.validChecker(readAccountInfo(), storageFile.getNextAccount())){
					break;
				}
				NumsOfAccount--;
			}
			
			/*if an account is found*/
			if(NumsOfAccount!=0){                                   
				dispose();
				/*enter TeacherGUI when the account type is 1*/
				if(getAccountType() == 1){                         
					TeacherGUI.createAndShowGUI();    
					}
				/*enter StudentGUI when the account type is 2*/
				if(getAccountType() == 2){                          
					
					/* Student GUI should be shown here*/
			        SpectroLab spectroLabGUI = new SpectroLab();
			        spectroLabGUI.createAndShowGUI(); 
					}
			}
			/* if no account is found, display the input account is invalid*/
			else                                                    
				JOptionPane.showMessageDialog(null, "invalid ID or password");	
		}
	}
	/**
	 * This class is a private class that implements ActionListener. After clicking Exit 
	 * button, the application is closed.  
	 * @see    ActionListener
	 * @author Chao Han
	 *
	 */
	private class ExitHandler implements ActionListener{

		@Override
		public void actionPerformed(ActionEvent e) {
			System.exit(0);
		}
		
	}
}