package biology.spectro.collaboration;

import biology.spectro.presentation.SpectroEditor;
import flatscape.util.Debug;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.Window;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextArea;

import manifold.collaboration.BeanControl;
import manifold.collaboration.RemoteCommandListener;
import manifold.collaboration.RemoteViewCommandListener;
import manifold.domain.CommandEvent;
import manifold.domain.DomainControl;
import manifold.presentation.Editor;
import manifold.presentation.PresentationControl;
import manifold.presentation.ViewCommandEvent;

public class SpectroLab extends JPanel 
implements RemoteCommandListener, RemoteViewCommandListener {
	
	protected BeanControl controller;
	protected Editor editor;
	private   Debug debug = new Debug();
	
	public SpectroLab(){
		super(new FlowLayout(FlowLayout.CENTER, 0, 0));
		
		

		// use the default implementations
		/*PresentationControl presentation = new PresentationControl();
		controller = new BeanControl(new DomainControl(), presentation);
		editor = SpectroEditor.createEditor(presentation);
		add(editor.getView());
		editor.createNewDocument();
		debug.debugOn();*/
	}
	/**
	 * Registers a RemoteCommandListener that is interested in receiving the events
	 * related to processing the Commands.
	 * @param rcl listener object's reference
	 */
	public void addRemoteCommandListener(RemoteCommandListener rcl) {
		controller.addRemoteCommandListener(rcl);
	}
	/**
	 * Registers a RemoteViewCommandListener that is interested in receiving the events
	 * related to processing the Commands.
	 * @param rvcl listener object's reference
	 */
	public void addRemoteViewCommandListener(RemoteViewCommandListener rvcl) {
		controller.addRemoteViewCommandListener(rvcl);
	}
	public Component getView() 
	{
		return editor.getView();
	}

	/**
	 * Receives a command from a <em>remote</em> source
	 * and passes it to its coupling state object.
	 * @param cmd command to be processed
	 */
	public void processRemoteCommand(CommandEvent cmd) {
		controller.processRemoteCommand(cmd);
	}
	/**
	 * Receives a view command from a <em>remote</em> source
	 * and passes it to its coupling state object.
	 * @param cmd command to be processed
	 */
	public void processRemoteViewCommand(ViewCommandEvent vcmd) {
		controller.processRemoteCommand(vcmd);
	}
	/**
	 * Deregisters a RemoteCommandListener.
	 * @param rcl listener object's reference
	 */
	public void removeRemoteCommandListener(RemoteCommandListener rcl) {
		if (rcl == this) {
			rcl = controller;
		}
		controller.removeRemoteCommandListener(rcl);
	}
	/**
	 * Deregisters a RemoteCommandListener.
	 * @param rvcl listener object's reference
	 */
	public void removeRemoteViewCommandListener(RemoteViewCommandListener rvcl) {
		if (rvcl == this) {
			rvcl = controller;
		}
		controller.removeRemoteViewCommandListener(rvcl);
	}
	

	public void createAndShowGUI(){
		JFrame f = new JFrame("Spectrophotometry Software Engineering 2012 Group 5");
        Debug debug = new Debug();
        
		f.addWindowListener(new WindowAdapter() {
	         public void windowClosing(WindowEvent e) {
	             System.exit(0);
	         }
	    });
		
		PresentationControl presentation = new PresentationControl();
		BeanControl controller_ = new BeanControl(new DomainControl(), presentation);
		Editor editor_ = SpectroEditor.createEditor(presentation);
		f.add(editor_.getView());
		editor_.createNewDocument();
		debug.debugOn();
		
		/*Create Instruction Panel and Show it*/
		JPanel infoBoardPanel = new JPanel();
		FileMgmt infoFile = new FileMgmt();
		JTextArea infoBoardField = new JTextArea(infoFile.readFile("src/biology/spectro/resources/InfoBoard.txt"),20,20);
		infoBoardPanel.add(infoBoardField);
		infoBoardField.setEditable(false);
		infoBoardPanel.setVisible(true);
		infoBoardField.setLineWrap(true);
		infoFile.closeFile("src/biology/spectro/resources/InfoBoard.txt");
		f.add(infoBoardPanel, BorderLayout.WEST);
		
		//f.getContentPane().add(new SpectroLab());
		f.pack();
		f.setVisible(true);
	}
}