package biology.spectro.domain;
 
import flatscape.domain.TextGlyph;
import flatscape.domain.Transform2D;
import jess.JessException;
import manifold.domain.Behavior;
import manifold.domain.Glyph;
import manifold.domain.TransformEvent;
import manifold.domain.TransformListener;

/**
 * This class is used to process rotation of dials
 * @author Chao Han
 *
 */
public class DialWatcher implements Behavior, TransformListener {
	private SpectroBehavior  spectro;
	private TextGlyph        waveReading;
	
	private double  waveTemp;
	private double  lightTemp;
	private double  darkTemp;
	private double  lastReading = 340.0D;
	private double  lastAngle = 0.75D;
	private double  lastMeterReading = 11.5D;
	private double  rotWave;
	private double  initialAngle;
	private double  finalAngle;
	private int     index = 0;

	/**
	 * adds targets
	 */
	public void addTarget(String componentName, Object object) {
		if (componentName.equals("spectroBehavior")) {
			spectro = ((SpectroBehavior)object);
  		} else if (componentName.equals("waveReading")) {
  			waveReading = ((TextGlyph)object);
  		} else {
  			System.err.println("SwitchWatcher.addTarget: Unknown target " + componentName);
  		}
	}
	public void setSelectOccured(TransformEvent transformEvent) {
		
	}

	/**
	 * sets index to 1 when grasp happens
	 */
	public void graspOccured(TransformEvent event) {
		if (index == 0) {
  		initialAngle = spectro.getLastMovedAngle();
  		index += 1;
  		}
  	}
	
	/**
	 * sets finalAngle and index to 0;
	 */
	public void releaseOccured(TransformEvent event) {
		//String str = (String)event.source_.getGlyphProperty("glyph.dialType").getValue();
		finalAngle = spectro.getLastMovedAngle();
		
		if (initialAngle == finalAngle) {
			try{
				spectro.dialHelpNecessary();
			} catch (JessException e) {
				System.out.println("Dial exception" + e);
			}
  		}
		index = 0;
	}

	/**
	 * process rotation
	 */
	public void transformOccured(TransformEvent event) {
		lastAngle = spectro.getLastMovedAngle();
		lastMeterReading = spectro.getReading();
		
		//Glyph glyph = event.source_;
		
		/*display enlarged view, if rotation happens*/
		spectro.setEnlargedView(true);
		
		String dialType = (String)event.source_.getGlyphProperty("glyph.dialType").getValue();
		
		double angle;
		boolean isSwitchOn = spectro.isSwitchedOn();
		if (isSwitchOn) {                                                     /*if device is on*/
			double rotationAngle;
			
			/*process light dial rotation*/
			if (dialType.equals("light")) {                                   /*if light dial is rotated*/
				rotationAngle = ((Transform2D)event.transform_).getRotation();  
				
                /* rotation angle should between 0 and 2*pi */
				if (rotationAngle > 0.0D) {                                   /*if rotation angle is positive*/
					rotationAngle %= 6.283185307179586D;                      /* 2pi */
				} else {                                                      /*if rotation angle is less than 0*/
					while (rotationAngle < 0.0D) {                            /*keep adding 2pi till it is positive*/
  					rotationAngle += 6.283185307179586D;
  					}
				}
			
				angle = rotationAngle - lightTemp;
				angle = makePIAdjustments(angle);                             /*angle is between -pi and pi*/
				
				lastAngle += angle / 15.0D;                                   /*sets angle needle moved when rotating light dial*/
				spectro.takeLightDialAction(angle / 15.0D);                   /*sets delta angle*/
				recordLightTemp(rotationAngle);                               /*sets LightLemp = rotationAngle*/
				spectro.recordLastMovedAngle(lastAngle);

				lastMeterReading += angle * 100.0D / 32.986722862692829D;   
				spectro.setReading(lastMeterReading);                         /*spectro.reading = lastMeterReading*/
  			    System.out.println("The reading is" + lastMeterReading);
  			}
			
			/*process dark dial rotation*/
			if (dialType.equals("dark")) {                                    /*if dark dial is rotated*/
				rotationAngle = ((Transform2D)event.transform_).getRotation();
				
				/*rotation angle is between 0 and 2*pi */
				if (rotationAngle > 0.0D) {
					rotationAngle %= 6.283185307179586D;
				} else {
					while (rotationAngle < 0.0D)
  					rotationAngle += 6.283185307179586D;
				}
				
				angle = rotationAngle - darkTemp;
				angle = makePIAdjustments(angle);                              /*adjust angle to interval(-pi,pi) */
				
				lastAngle += angle / 15.0D;
				spectro.takeDarkDialAction(angle / 15.0D);                      
				recordDarkTemp(rotationAngle);                                 /*sets darkTemp = rotationAngle*/
				spectro.recordLastMovedAngle(lastAngle);
				lastMeterReading += angle * 100.0D / 32.986722862692829D;
  			    spectro.setReading(lastMeterReading);
  			    System.out.println("The reading is" + lastMeterReading);
  		    }
		} else {                                                               /*if the device is off*/
			try {
				spectro.switchNotOn();                                         /*display switch-not-on*/
			} catch (JessException e) {                                        
				System.out.println(" Exception in JESS !!!" + e);
  	 	    }
  	    }
		
		/*process wavelength rotation*/
		if (dialType.equals("wave")) {                                         /*if wavelength dial is rotated*/
			double d2 = spectro.getLastMovedAngle();
  		    rotWave = ((Transform2D)event.transform_).getRotation();
  		    
  		    /*rotWave is between 0 and 2*pi */
  		    if (rotWave > 0.0D) {
  		    	rotWave %= 6.283185307179586D;                                 /*2pi*/
  		    } else{
  			    while (rotWave < 0.0D) {
  		        rotWave += 6.283185307179586D;
  	            }
  		    }
  		    
  		    angle = rotWave - waveTemp;
  		    
  		    
  		    if (angle < -3.14D) {                                              /*if angle is smaller than -pi*/
  		    	lastReading += 100.0D;                                         /*lastReading + 100.0D*/
            } else if (angle > 3.14D) {                                        /*if angle larger than pi */
            	lastReading -= 100.0D;                                         /*lastReading - 100.0D*/
  		    }
  		    
  		    Integer wavelength = new Integer((int)(lastReading +                
  		    		angle * 400.0D / 25.132741228718345D));
 
  		    if ((wavelength.intValue() >= 340) &&                              /*if 340<= wavelength <= 740*/
  		    		(wavelength.intValue() <= 740)) {
  		    	waveReading.putPropertyValue("text", 
  		    			new String(wavelength.toString()));
  		    } else if (wavelength.intValue() < 340) {                          /*if wavelength <340*/
  		    	try{
  		    		spectro.wrongDirectionRotation();                          /*wrong direction warning*/
  		    	} catch (JessException e) {
  		    		System.out.println("Wave dial exception!!!" + e);           
  		    	}
  		    }
  		    
  		    lastReading += angle * 400.0D / 25.132741228718345D;
  		    recordWaveTemp(rotWave);
  		    
  		    if (isSwitchOn){
  		    	spectro.takeWaveDialAction(angle);
  		    }
  	    }
	}
	
	/**
	 * sets waveTemp
	 * @param temp temp value of wavelength
	 */
	public void recordWaveTemp(double temp) {
		waveTemp = temp;
  	}
	
	/**
	 * sets lightTemp 
	 * @param temp temp value of light dial
	 */
	public void recordLightTemp(double temp) {
		lightTemp = temp;
	}
	
	/**
	 * sets darkTemp
	 * @param temp temp value of darkdial
	 */
	public void recordDarkTemp(double temp) {
		darkTemp = temp;
	}
	
	/**
	 * This method is used to adjust angle (-pi, pi);
	 * @param rotatdeAngle 
	 * @return rotatedAngle angle after rotation
	 */
	public double makePIAdjustments(double rotatdeAngle) {
		
		if (rotatdeAngle < -3.14D) {                           /*if angle is less than minus pi*/
			rotatdeAngle += 6.283185307179586D;                /*add 2pi*/
			return rotatdeAngle;
		} else if (rotatdeAngle > 3.14D) {                     /*if angle is larger than pi*/
  	    	rotatdeAngle -= 6.283185307179586D;                /*sub 2pi*/
  	    	return rotatdeAngle;
  	    } else{
  	    	return rotatdeAngle;  
  	    }
	}
}
