package biology.spectro.domain;
 
import biology.domain.LabObject;
import flatscape.domain.Transform2D;
import manifold.domain.CreateGlyphCommand;
import manifold.domain.GlyphProperty;
 
public class LargeNeedle extends LabObject {
	protected float minAngle = 0.5F;
	protected float maxAngle = 2.68F;

	/**
	 * constructor for class LargeNeedle
	 * @param cmd
	 * @param transform
	 */
	public LargeNeedle(CreateGlyphCommand cmd, Transform2D transform) {
		super(cmd, transform); 	
		addGlyphProperty("LargeNeedle.minAngle", 
				new GlyphProperty(new Float(minAngle),
						"minimum rotation angle of the LargeNeedle", false));
		addGlyphProperty("LargeNeedle.maxAngle", 
				new GlyphProperty(new Float(maxAngle), 
						"maximum rotation angle of the LargeNeedle", false));
  
	}	
	
	/**
	 * rotate needle
	 */
	public void transform(Object obj) {
		Transform2D needleRotation = (Transform2D)obj;
		
		/*sets needle motion between minAngle and maxAngle*/
		if ((needleRotation.getRotation() >= minAngle) && 
				(needleRotation.getRotation() <= maxAngle)) {
			super.transform(obj);
		} else if (needleRotation.getRotation() < minAngle) {
			needleRotation.setRotation(minAngle);
			super.transform(needleRotation);
		} else if (needleRotation.getRotation() > maxAngle) {
			needleRotation.setRotation(maxAngle);
			super.transform(needleRotation);
		}
	}
	
	/**
	 * put property value
	 */
	public synchronized void putPropertyValue(String componentName, Object object) {
		super.putPropertyValue(componentName, object);
		if (componentName.equals("LargeNeedle.minAngle")) {
			minAngle = ((Float)object).floatValue();
		}
		
		if (componentName.equals("LargeNeedle.maxAngle")) {
			maxAngle = ((Float)object).floatValue();
		}
	}
}