/* 
 * Needle.java
 * The needle is a part of the absorbance meter
 *  
 * Version: 1.1.1
 * 
 * Date: 12-12-08
 * 
 * Copyright: Bingbing Xu, .....
 * */
package biology.spectro.domain;

import biology.domain.LabObject;
import flatscape.domain.Transform2D;
import manifold.domain.CreateGlyphCommand;
import manifold.domain.Glyph;
import manifold.domain.GlyphProperty;

public class Needle extends LabObject {
	protected float minAngle_ = 0.5F;
	protected float maxAngle_ = 2.68F;

  
	/**
	 * Constructor for class Needle, add properties such as minAngle and 
	 * maxAngle.  
	 * @param cmd
	 * @param transform
	 */
	public Needle(CreateGlyphCommand cmd, Transform2D transform) {
		super(cmd, transform);
		addGlyphProperty("needle.minAngle", 
				new GlyphProperty(new Float(minAngle_), 
						"minimum rotation angle of the needle", false));
		addGlyphProperty("needle.maxAngle", 
				new GlyphProperty(new Float(maxAngle_), 
						"maximum rotation angle of the needle", false));
		addGlyphProperty("glyph.toolTip",
				new GlyphProperty(new String("needle"), 
						"Tool tip for the glyph", true));
  }
	/**
	 * spins needle
	 */
	public void transform(Object object) {
		Transform2D needle = (Transform2D)object;
		
		if ((needle.getRotation() >= minAngle_) && 
				(needle.getRotation() <= maxAngle_)) {                  /*minAngle<needle<maxAngle*/
			super.transform(object);
		} else if (needle.getRotation() < minAngle_) {                  /*needle < minAngle*/
			needle.setRotation(minAngle_);
			super.transform(needle);
		} else if (needle.getRotation() > maxAngle_) {                  /*needle > maxAngle*/
			needle.setRotation(maxAngle_);
			super.transform(needle);
		}
	}

	/**
	 * grasp movement
	 */
	public void grasp() {
		super.grasp();
	}
	
	/**
	 * put properties value for needle
	 */
	public synchronized void putPropertyValue(String propertyValue, Object obj){
		super.putPropertyValue(propertyValue, obj);
		if (propertyValue.equals("needle.minAngle")) {
			minAngle_ = ((Float)obj).floatValue();
		}
		if (propertyValue.equals("needle.maxAngle")) {
			maxAngle_ = ((Float)obj).floatValue();
		}
	}
}
