package biology.spectro.domain;
/**
 * this class includes operations under state(off,occupied,open)
 * because the switch is off, the measure method does nothing and 
 * reading doesn't change. 
 * @author Chao Han
 *
 */
public class OffOccupiedOpen extends SpectroState {
	/**
	 * Constructor 
	 * @param state
	 */
	public OffOccupiedOpen(SpectroBehavior state){
		super(state);
	}

	/**
	 * measures test tube under state OffOccupiedOpen. Because switch is
	 * off, reading doesn't change. 
	 * @return number original value
	 */
	public double measure() {
		System.out.println("measuring while off, occupied and open");
		return number;
	}

	/**
	 * flips switch then state changes from OffOccupiedOpen to
	 * OnOccupiedOpen
	 */
	public void flipSwitch(){
		setState(new OnOccupiedOpen(spectro));
	}

	/**
	 * flips lid then state changes from OffOccupiedOpen to 
	 * OffOccupiedClosed
	 */
	public void flipLid(){
		setState(new OffOccupiedClosed(spectro));
	}
	
	/**
	 * removes test tube from sample holder then state changes from 
	 * OffOccupiedOpen to OffUnoccupiedOpen
	 */
	public void removeTestTube() {
		setState(new OffUnoccupiedOpen(spectro));
	}
}
