package biology.spectro.domain;

/**
 * this class includes operations under state(off,unoccupied,closed), 
 * which is the original state of the device. 
 * because the switch is off, the measure method does nothing and 
 * reading doesn't change. 
 * @author Chao Han
 *
 */
public class OffUnoccupiedClosed extends SpectroState {
	public OffUnoccupiedClosed(SpectroBehavior state){
		super(state);
	}
	
	/**
	 * measures test tube under state OffUnoccupiedClosed, because the 
	 * device is off, measure does nothing. 
	 * @return number original value
	 */
	public double measure(){
		System.out.println("measuring while off, unoccupied and closed");
		return number;
	}
 
	/**
	 * flips switch, then state changes to OnUnoccupiedClose
	 */
	public void flipSwitch(){
		setState(new OnUnoccupiedClosed(spectro));
	}
 
	/**
	 * flips lid, then state changes to OffUnoccupiedOpen
	 */
	public void flipLid() {
		setState(new OffUnoccupiedOpen(spectro));
	}
}
