package biology.spectro.domain;

/**
 * this class includes operations under state(Off,Unoccupied,Open)
 * because the switch is off, the measure method does nothing and 
 * reading doesn't change. 
 * @author Chao Han
 *
 */
public class OffUnoccupiedOpen extends SpectroState {
	/**
	 * Constructor
	 * @param state
	 */
	public OffUnoccupiedOpen(SpectroBehavior state) {
		super(state);
	}

	/**
	 * measures test tube, which does nothing
	 * @return  measure original value
	 */
	public double measure(){
		System.out.println("measuring while off, unoccupied and open");
		return this.number;
	}
	
	/**
	 * flips switch, then state changes to OnUnoccupiedOpen
	 */
	public void flipSwitch() {
		setState(new OnUnoccupiedOpen(spectro));
	}
	
	/**
	 * flips lid, then state changes to OffUnoccupiedClosed
	 */
	public void flipLid(){
		setState(new OffUnoccupiedClosed(spectro));	
	}
	
	/**
	 * inserts test tube into sample holder, then state chagnes to 
	 * OffOccupiedOpen
	 */
	public void setTestTube(){
		setState(new OffOccupiedOpen(spectro));
	}
}