package biology.spectro.domain;

import biology.domain.LabObject;
import flatscape.domain.Transform2D;
import java.awt.Color;
import java.awt.geom.Rectangle2D;
import manifold.domain.CreateGlyphCommand;
import manifold.domain.GlyphProperty;

public class PowerSwitch extends LabObject {
	/**
	 * adds properties
	 * @param cmd
	 * @param trans
	 */
	public PowerSwitch(CreateGlyphCommand cmd, Transform2D trans){
		super(cmd, trans);
		addGlyphProperty("switch.state", 
				new GlyphProperty(new Boolean(false), 
						"Switch state ON/OFF", true));
		addGlyphProperty("glyph.width", 
				new GlyphProperty(new Double(getBounds().getWidth()), 
						"Width of the rectangle", true));
		addGlyphProperty("glyph.height", 
				new GlyphProperty(new Double(getBounds().getHeight()), 
						"Height of the rectangle", true));
		addGlyphProperty("fill.color", 
				new GlyphProperty(new Color(0, 0, 0), 
						"Color of the rectangle", true));
		addGlyphProperty("glyph.toolTip",
				new GlyphProperty(new String("Power Switch"),
						"Tool tip for the glyph", true));
	}

	/**
	 * Operation of grasping
	 */
	public void grasp() {
		boolean switchState = 
				((Boolean)getPropertyValue("switch.state")).booleanValue();
		putPropertyValue("switch.state", new Boolean(!switchState));
		Transform2D switchPos = (Transform2D)transform_;
		if (!switchState) {
			switchPos.setRotation(90.0D);                    /*rotation degree for closed state*/
		} else {
			switchPos.setRotation(255.0D);                   /*rotation degree for open state*/
		}
		transform(switchPos);
	}
	
	/**
	 * puts Property value for switch
	 */
	public synchronized void putPropertyValue(String objname, Object obj) {
		super.putPropertyValue(objname, obj);
		double widthOrHeight;
		Rectangle2D.Float powerSwitch;
		if (objname.equals("glyph.width")) {
			widthOrHeight = ((Double)obj).doubleValue();
			powerSwitch = (Rectangle2D.Float)getPropertyValue("glyph2D.bbox");
			powerSwitch.setRect(powerSwitch.getX(), powerSwitch.getY(),
					widthOrHeight, powerSwitch.getHeight());
			putPropertyValue("glyph2D.bbox", powerSwitch);
		}
		if (objname.equals("glyph.height")) {
			widthOrHeight = ((Double)obj).doubleValue();
			powerSwitch = (Rectangle2D.Float)getPropertyValue("glyph2D.bbox");
			powerSwitch.setRect(powerSwitch.getX(), powerSwitch.getY(),
					powerSwitch.getWidth(), widthOrHeight);
			putPropertyValue("glyph2D.bbox", powerSwitch);
		}
	}
}
