package biology.spectro.domain; 

import biology.domain.BackButton;
import biology.domain.ExitButton;
import biology.domain.HelpButton;
import biology.domain.NextButton;
import flatscape.domain.Transform2D;
import java.awt.Color;
import java.io.PrintStream;
import java.util.Vector;
import jess.JessException;
import jess.MiscFunctions;
import jess.ReflectFunctions;
import jess.Rete;
import jess.Value;
import manifold.collaboration.BeanControl;
import manifold.domain.Behavior;
import manifold.domain.CommandHistory;
import manifold.domain.Glyph;
import manifold.domain.commands.SetPropertyCommand;
import manifold.domain.commands.UndoRedoCommand;
import manifold.presentation.PresentationControl;

/**
 * This class, which implements interface Behavior, is used to implement animation 
 * of components.  
 * 
 * @version  1.01   7 Dec 2012
 * @author   Chao Han
 */
public class SpectroBehavior implements Behavior {
	
	/*current state of this software*/
	private SpectroState state;
    
	/*components of the Spectrophotometer*/
	private Needle          needle;
    private LargeNeedle     largeNeedle;
    private Glyph           lamp;
    private TestTubeFigure  testTube;
    private SampleHolder    sampleHolder;
    private AbsorbanceMeter absorbanceMeter;
    
    /*buttons of this software*/
    private NextButton nextButton;
    private BackButton backButton;
    private ExitButton exitButton;
    private HelpButton helpButton;
    
    /*text field where display instructions*/
    private InstructionText iText;
    
    /*states of components*/
    private boolean lampOn;
    private boolean isHolderOccupied;
    private boolean isLidOpen;
    private boolean isCovered;
    private boolean isEnlargedView = false;
    private boolean isSwitchedOn;
    private boolean helpAsked;
    
    /*data related to absorbanceMeter display */
    private int     stage = 1;
    private double  val;
    private double  reading = 11.5D;
    private int     oldSize;
    private double  angle = 0.75D;
    private double  delta;
    private double  bias;
    
    private Rete rete;
    private UndoRedoCommand urc = new UndoRedoCommand();
    private String[] instructions;
    
    /**
     * Constructor for class SpectroBehavior
     * @throws JessException 
     */
    public SpectroBehavior() throws JessException {
    	
    	/*initiate the state of the device to the OffUnccupiedClosed state*/ 
    	state = new OffUnoccupiedClosed(this);
    	String str = System.getProperty("line.separator");
    	instructions = new String[4];
    	biology.domain.InitApplet.testString = "test";
 
    	/*content of instructions*/
    	instructions[0] = "Begin by setting the Wavelength. " +
    			"For this, click on the wave dial " +
    			"to obtain a top view.             ";
    	instructions[0] = instructions[0].concat("Rotate " +
    			"the wavelength control until the desired " +
    			"wavelength is indicated by the wavelength scale.   ");
    	instructions[0] = instructions[0].concat("Click " +
    			"on the switch to turn it on.                       " +
    			"                                          ");
        instructions[0] = instructions[0].concat("Bring the meter" +
        		" needle to infinity on the absorbance scale " +
        		"by adjusting the dark control.          ");
       
        instructions[1] = "Great!!! Click on the " +
        		"sample holder to open it.        " +
        		"                                            ";
        instructions[1] = instructions[1].concat("Now choose " +
        		"the test tube with the transparent solution and " +
        		"place it in the sample holder.          ");
        instructions[1] = instructions[1].concat("THEN CLOSE " +
        		"THE SAMPLE HOLDER CLICKING IT SHUT.            " +
        		"                                         ");
        instructions[1] = instructions[1].concat("Rotate " +
        		"the light control until the meter " +
        		"indicates zero on the absorbance scale.                   ");
        instructions[1] = instructions[1].concat("This procedure " +
        		"calibrates the instrument relative to the blank." +
        		"                                    ");                                
        
        instructions[2] = "Wonderful!!! Now open the sample holder" +
        		" and drop the transparent test tube back into" +
        		" the rack.     ";
        instructions[2] = instructions[2].concat("Click " +
        		"on any of the other sample test tubes" +
        		" and drop it into the sample holder as before.          ");
        instructions[2] = instructions[2].concat("Now " +
        		"take down the readings of transmittance" +
        		"and absorbtion only after closing the sample holder.   ");
        instructions[2] = instructions[2].concat("Repeat " +
        		"the above process for all the test tubes. " +
        		"Close the window after you are done.              ");
        
        rete = new Rete();
        rete.addUserpackage(new MiscFunctions());
        rete.addUserpackage(new ReflectFunctions());
        
        /*executing commands*/
        rete.executeCommand("(batch http://www.ece.rutgers.edu/" +
        		"\\~marsic/groupware/ViBE/scriptlib.clp)");
        rete.executeCommand("(batch http://www.ece.rutgers.edu/" +
        		"\\~marsic/groupware/ViBE/Vibe.clp)");
        rete.executeCommand("(watch all)");
        rete.executeCommand("(reset)");
        rete.executeCommand("(assert (wavedial-unused) )");
        rete.executeCommand("(assert (show-front) )");
        rete.run();
    }
 
    /**
     * Adds the components
     * @param componentName  the Name of the component
     *        component      the component
     */
    public void addTarget(String componentName, Object component){
    	switch (componentName){
    	case "needle":
    		System.out.println("Adding the target to " +
    				"spectroBehavior..needle");
    		needle = ((Needle)component);
    		break;
    		
    	case "largeNeedle":
    		System.out.println("Adding the target to " +
    				"spectroBehavior..largeNeedle");
    		largeNeedle = ((LargeNeedle)component);
    		break;
    		
    	case "pilotLamp":
    		System.out.println("Adding the target to " +
    				"spectroBehavior..pilotLamp");
    		lamp = ((Glyph)component);
    		break;
    		
    	case "absorbanceMeter":
    		System.out.println("Adding the target to " +
    				"spectroBehavior..absorbanceMeter");
    		absorbanceMeter = ((AbsorbanceMeter)component);
    		break;
    		
    	case "sampleHolder":
    		System.out.println("Adding the target to " +
    				"spectroBehavior..sampleHolder");
    		sampleHolder = ((SampleHolder)component);
    		break;
    		
    	case "nextButton":
    		System.out.println("Adding the target to " +
    				"spectroBehavior..nextButton");
    		nextButton = ((NextButton)component);
    		break;
    		
    	case "backButton":
    		System.out.println("Adding the target to " +
    				"spectroBehavior..nextButton");
    		backButton = ((BackButton)component);
    		break;
    		
    	case "instructionText":
    		System.out.println("Adding the target to " +
    				"spectroBehavior..instructionText");
    		iText = ((InstructionText)component);
    		iText.putPropertyValue("text", new String(instructions[0]));
    		break;
    		
    	case "exitButton":
    		System.out.println("Adding the target to " +
    				"spectroBehavior..exitButton");
    		exitButton = ((ExitButton)component);
    		break;
    		
    	case "helpButton":
    		System.out.println("Adding the target to " +
    				"spectroBehavior..helpButton");
    		this.helpButton = ((HelpButton)component);
    		break;
    		
    	default:
    		System.err.println("SpectroBehavior.addTarget: " +
    				"Unknown target " + componentName);
    		break;
    	}
    }
    
    /**
     * Sets state
     * @param spectroState state of Spetro
     */
    public void setState(SpectroState spectroState){
    	if (spectroState != null){
    		state = spectroState;
    	}
    }
    /**
     * Sets isEnlargedView
     * @param enlargedView status of enlarged view (1 activated 0 non-activated)
     */
    public void setEnlargedView(boolean enlargedView){
    	isEnlargedView = enlargedView;
    	absorbanceMeter.putPropertyValue("absorbanceMeter.needEnlargedView", 
    			new Boolean(enlargedView));
    	largeNeedle.putPropertyValue("glyph.visible", 
    			new Boolean(enlargedView));
    }
    
    /**
     * Sets status of buttons
     * @param buttonStatus button is activated or not
     */
    public void setButtonStatus(boolean buttonStatus) {
    	nextButton.putPropertyValue("button.isActivated",
    			new Boolean(buttonStatus));
	}
    
    /**
     * The action of flipping the switch. if switch is on, 
     * lamp changes from gray to green. 
     * @param isSwitchOn the status of the switch
     */
    public void flipSwitch(boolean isSwitchOn)
    {
    	Color lampColor = null;     
    	state.flipSwitch();
    	lampOn = isSwitchOn;                              /*sets status of the lamp*/                  
    	isSwitchedOn = isSwitchOn;                        /*sets status of the switch*/
    	
    	/*process status of switch*/                     
    	if (isSwitchOn){ 
    		lampColor = new Color(0, 255, 0);           /*changes lampColor to green(R,G,B)*/
    		try{
    			rete.retractString("(switch-not-on)");
    		} catch (JessException e){
    			System.out.println(e);
    		}
    	} else {
    		lampColor = new Color(125, 125, 125);       /*changes lampColor to Gray*/
    	}
    	SetPropertyCommand cmd = new SetPropertyCommand(lamp.getId(), 
    			"fill.color", lampColor);
    	BeanControl.presentation_.fireCommand(cmd);
    }
    
    /**
     * Returns the status of the switch
     * @return isSwithedOn the status of the switch
     */
    public boolean isSwitchedOn(){
    	return isSwitchedOn;
    }
    
    /**
     * Sets testTube
     * @param testTubeFigure
     */
    public void setTestTube(TestTubeFigure testTubeFigure) {
    	testTube = testTubeFigure;
    	isHolderOccupied = true;
    	checkTestTubeContents();
    }
    
    /**
     * Handle the relationship between status of lid and needle
     * <p>
     * The status of lid has effects on the result of the experiment. If the lid is open, extra light
     * will enter the sample holder which lowers the result
     * @param lidStatus the status of lid
     * @throws JessException
     */
    public void setLidOpen(boolean lidStatus) throws JessException {
    	isLidOpen = lidStatus;                             
    	
    	/*sets bias of the needle based on the status of the lid*/
    	if (isLidOpen){
    		bias = 0.05D;
    	} else{ 
    		bias = -0.05D;
    	}
    	
    	/*recalculate the angle of the needle*/
    	angle += bias; 
    	transformNeedle(angle + delta);
    	
    	if (testTube != null){
    		if ((isHolderOccupied == true) && (!isLidOpen)){                     /*if sample holder is occupied and lid is close*/
    			testTube.putPropertyValue("glyph.visible", 
    					new Boolean(false));
    			rete.retractString("(lid-open)");
    		} else if ((isHolderOccupied == true) && (isLidOpen == true)) {      /*if sample holder is occupied and lid is open*/   
    			testTube.putPropertyValue("glyph.visible", 
    					new Boolean(true));
    			rete.executeCommand("(assert (lid-open))");
    		}
    	}
    	state.flipLid();
    }
    
    /**
     * Gets test tube
     * @return testTube test tube
     */
    public TestTubeFigure getTestTube() {
    	return testTube;
    }
    
    /**
     * removes the test tube from sample holder
     */
    public void removeTestTube(){
    	isHolderOccupied = false;
    	testTube = null;
    	state.removeTestTube();
    }
    
    /**
     * Handles rotation of Dark Dial
     * @param dialAction angle of rotation
     */
    public void takeDarkDialAction(double dialAction){
    	delta = dialAction;
    	state.takeDialAction();
    }
    
    /**
     * Handles rotation of Light Dial
     * @param dialAction angle of rotation
     */
    public void takeLightDialAction(double dialAction){
    	delta = dialAction;
    	state.takeDialAction();
    }
    
    /**
     * Handles rotation of Wavelength Dial
     * @param dialAction angle of rotation
     */
    public void takeWaveDialAction(double dialAction) {
    	delta = dialAction;
    	state.takeDialAction();
    }
    
    /**
     * set the status of test tube in sample holder
     * @param paramBoolean
     */
    public void setCovered(boolean testTubeStatus){
    	isCovered = testTubeStatus;
    	if ((!isLidOpen) && (isCovered == true)){        /*if Lid is close and test tube in sample holder is covered*/
    		testTube.putPropertyValue("testTube.isCovered",
    				new Boolean(true));
    	}
    }

    /**
     * Sees whether the test tube can be inserted into the sample hold
     * <p>
     * This makes the software reasonable. If the lid is close or a 
     * test tube has already been in the sample holder, the selected 
     * test tube cannot be placed.  
     */
    public boolean seeIfTestTubeCanBePlaced(){
    	if ((!isHolderOccupied) && (isLidOpen == true)){ /*if sample holder is not occupied and lid is open*/
    		return true;
    	}
    	return false;
    }
    
    /**
     * records angle of the last move.
     * @param lastMoveAngle angle of the last move
     */
    public void recordLastMovedAngle(double lastMoveAngle){
    	angle = lastMoveAngle;
    }
    
    /**
     * Returns angle of last move
     * @return angle angle of last move
     */
    public double getLastMovedAngle() {
    	return angle;
    }
    
    /**
     * Sets absorbanceMeter reading
     * @param meterReading reading of absorbanceMeter
     */
    public void setReading(double meterReading) {
    	reading = meterReading;
    	absorbanceMeter.putPropertyValue("absorbanceMeter.transmittanceReading",
    			new Double(reading));
    }
    
    /**
     * Sets instructions for stage
     * @param stageNo
     */
    public void setInstructionForStage(int stageNo) {
    	boolean isButtonClicked = ((Boolean)nextButton.
    			getPropertyValue("button.isClicked")).booleanValue();
    	if (stageNo > 2) {
    		stageNo = 2;
    	}

    	if (isButtonClicked){
    		iText.putPropertyValue("text", new String(instructions[stageNo]));
    	}
    }
    
    /**
     * Sets instructions stage based on the position of the needle
     * <p>
     * @throws JessException
     */
    public void setInstructions() throws JessException {
    	oldSize = urc.getCommandHistory().getCommands().size(); 
    	
    	/* set instruction stage*/
    	if (((reading > 3.0D) && (reading < 8.0D)) || (reading < 1.0D)){            /*if reading < 1.0, or 3.0 < reading < 8.0*/
    		setWarningText2();                                                      /*incorrect position, and display waring text*/
    		try {
    			rete.executeCommand("(assert (fine-tuning-necessary) )");
    		} catch (JessException e1){
    			System.out.println("Exception in fune tuning" + e1);
    		}
    	} else if ((reading <= 3.0D) && (reading > 1.0D)){                          /*if 1.0 < reading <3.0 */
    		stage += 1;                                                             /*next stage*/
    		iText.putPropertyValue("text", new String(instructions[(stage - 1)]));  
    	} else if ((reading >= 97.0D) && (reading < 100.0D) && (stage == 2)){       /*in stage 2, if 97.0<reading <100.0*/
    		stage += 1;                                                             /*next stage*/
    		iText.putPropertyValue("text", new String(instructions[(stage - 1)]));
    	} else if ((reading > 99.0D) && (stage == 2)){                              /*in stage 2, if reading > 99.0*/
    		setWarningText2();                                                      /*incorrect position, and display warning text*/
    		try{
    			rete.executeCommand("(assert (fine-tuning-necessary) )");
    		} catch (JessException e2){
    			System.out.println("Exception in fune tuning" + e2);
    		}
    	}
    }
    
    /**
     * checks content of a test tube, and get the value
     */
    public void checkTestTubeContents(){
    	String color = ((String)testTube.getPropertyValue("testTube.contentInfo"))
    			.toString();
    	
    	/*get value of sample*/
    	val = ((Double)testTube.getPropertyValue("testTube.deflectionValue")).doubleValue();
    	
    	/*process while the device is switched on*/
    	if (isSwitchedOn){
    		if (((color.equals("colorless")) && (stage == 2)) || (stage == 3)){    /*in stage 2 or 3, "colorless" solution inserted*/ 
    			transformNeedle(val); 
    			angle = val;                                                       /*angle equals sample's value*/
    			setReading(45.871559633027516D * (val - 0.5D));                    /**/    
    		} else if ((color.compareTo("colorless") != 0) && (stage <= 2)){       /*if put color sample before stage 2*/
    			setWarningText1();                                                 /*incorrect operation, and display warning*/
    		}
    	}
    }
    
    /**
     * Warning text 1 for wrong stage
     */
    public void setWarningText1(){
    	SetPropertyCommand cmd = new SetPropertyCommand(iText.getId(), "text", 
    			"Please Click the Back button to finish the prev stage first");
    	BeanControl.presentation_.fireCommand(cmd);
    	backButton.putPropertyValue("backButton.isActivated", new Boolean(true));
    }
    
    /**
     * Warning text 2 for wrong needle position
     */
    public void setWarningText2(){
    	SetPropertyCommand cmd = new SetPropertyCommand(this.iText.getId(), "text", 
    			"You have not set the reading exactly!! " +
    			"Please click on the Back button to go back and set it again");
    	BeanControl.presentation_.fireCommand(cmd);
    	backButton.putPropertyValue("backButton.isActivated", new Boolean(true));
    }
    
    /**
     * Sets needles back to zero position, both small one and large one, at the same
     * time
     */
    public void setNeedleBackToZero() {
    	if (isSwitchedOn){
    		Transform2D needlePos = new Transform2D(145.5D, 447.0D, 1.0D, 
    				1.0D, 0.5D, 10.0D, 0.0D);
    		needle.transform(needlePos);
    		Transform2D largeNeedlePos = new Transform2D(165.5D, 297.0D, 1.0D,
    				1.0D, 0.5D, 75.0D, 0.0D);
    		largeNeedle.transform(largeNeedlePos);
    		angle = 0.5D;
    		setReading(0.0D);
    	}
    }
    
    /**
     * Gets reading
     * @return reading the value of sample
     */
    public double getReading(){
    	return reading;
    }
    
    /**
     * Sets whether a button can be clicked by measuring value
     * @param readingVal
     */
    public void setConditionSatisfied(double readingVal){
    	if ((readingVal > 3.0D) && (readingVal <= 4.0D) && (stage == 1)             /*if in stage 1, 3.0<reading<4.0*/
    			&& (testTube == null)){
    		if (!isHolderOccupied){                                                 /*if the sample holder is not occupied*/                               
    			setButtonStatus(true);                                              
    		}
    	} else if ((readingVal <= 3.0D) && (readingVal > 1.0D) && (stage == 1)){    /*if in stage 1, 1.0<readingVal<3.0*/
    		if (!isHolderOccupied) {                                                /*if the sample holder is not occupied*/
    			setButtonStatus(true);                                              /*button can be clicked*/
    		}		
    	} else if ((readingVal > 4.0D) && (stage == 1)){                            /*in stage 1, if reading is larger than 4.0*/                          
    		setButtonStatus(false);                                                 /*button cannot be clicked*/
    	} else if (readingVal <= 0.0D){                                             /*if reading is smaller than 0.0*/
    		try{
    			rete.executeCommand("( assert (over-shoot) )");                     /* over-shoot */
    		} catch (JessException e1){
    			System.out.println("over-shoot exception!!! " + e1);
    		}
    	} else if ((readingVal >= 97.0D) && (stage == 2)){                          /*in stage 2, if reading is larger than 97.0*/
    		if (testTube != null){                                                  /**/
    			String sampleColor = ((String)testTube.                             /*gets color of the sample*/
    					getPropertyValue("testTube.contentInfo")).toString();       /**/
    			
    			/*in stage 2, if it is the transparent sample, and reading is less than 100.0*/
    			if ((sampleColor.equals("colorless")) && (readingVal <= 100.0D)){    
    				setButtonStatus(true);                                          /*button can be clicked*/
    			}
    			
    			/*if the reading is more than 100.0*/
    			if (readingVal > 100.0D){                                           
    				setButtonStatus(false);                                         /*button cannot be clicked*/
    				try{
    					rete.executeCommand("( assert (over-shoot) )");             /* over-shoot*/
    				} catch (JessException e2){
    					System.out.println("over-shoot exception!!! " + e2);
    				}
    			}
    		}
    	} else if ((readingVal < 97.0D) && (stage == 2)){                           /*if, in stage 2, reading<97.0*/
    		setButtonStatus(false);                                                 /*button cannot be clicked*/
    	}
    }
    
    /**
     * 
     */
    public void undoLastCommand(){                                                  /**/
    	int i = urc.getCommandHistory().getCommands().size();
    	System.out.println("size is " + i);
    	
    	urc.execute(BeanControl.presentation_, i - oldSize);
    	
    	for (int j = 5; j < i; j++);
    }
    
    /**
     * Transform the position of needle
     */
    public void transformNeedle(double needleTrans){
    	Transform2D needlePos = new Transform2D(145.5D, 447.0D, 
    			1.0D, 1.0D, needleTrans, 10.0D, 0.0D);
    	needle.transform(needlePos);
    	Transform2D largeNeedlePos = new Transform2D(165.5D, 297.0D,
    			1.0D, 1.0D, needleTrans, 75.0D, 0.0D);
    	largeNeedle.transform(largeNeedlePos);
    	setReading(45.871559633027516D * (needleTrans - 0.5D));
    }
    /**
     * exit the program
     */
    public void callExit(){
    	System.exit(0);
    }
    
    /**
     * Switch on 
     * @throws JessException
     */
    public void switchNotOn() throws JessException {
    	rete.executeCommand("(assert (switch-not-on) )");
    	rete.run();
    }
    
    /**
     * asks for help
     * @throws JessException
     */
    public void askForHelp() throws JessException {
    	helpAsked = true;
    	rete.executeCommand("(assert (help-requested) )");
    	rete.run();
    	rete.executeCommand("(facts)");
    	int i = rete.executeCommand("(get-correct)").
    			intValue(rete.getGlobalContext());
    	int j = rete.executeCommand("(get-incorrect)").
    			intValue(rete.getGlobalContext());
    	System.out.println("The number of helpful helps ;-) is " + 
    			i + " and incorrect ones are " + j);
    }
    
    /**
     * dial help
     * @throws JessException
     */
    public void dialHelpNecessary() throws JessException {
    	rete.executeCommand("(assert (dial-difficulty) )");
    }
    
    /**
     * retract wavelength dial
     * @throws JessException
     */
    public void retractWaveDialUnUsed() throws JessException {
    	rete.retractString("(wavedial-unused)");
    }
    
    /**
     * test tube help
     * @throws JessException
     */
    public void testTubeHelpNecessary() throws JessException {
    	rete.executeCommand("(assert (testtube-place) )");
    }
    
    /**
     * processes wrong direction rotation
     * @throws JessException
     */
    public void wrongDirectionRotation() throws JessException {
    	rete.executeCommand("(assert (wrong-dir-rotate) )");
    }
    
    /**
     * this method measures and display value of test tube in the sample holder
     */
    public void measure() {
    	//double d = state.measure(); 
    	Transform2D needlePos = new Transform2D(145.5D, 447.0D, 1.0D, 1.0D, 
    			angle + delta, 10.0D, 0.0D);
    	needle.transform(needlePos); 
    	Transform2D largeNeedlePos = new Transform2D(165.5D, 297.0D, 1.0D, 1.0D, 
    			angle + delta, 75.0D, 0.0D);
    	largeNeedle.transform(largeNeedlePos);
    }
}