package biology.spectro.domain;

import flatscape.domain.Transform2D;
import java.awt.Toolkit;
import jess.JessException;
import manifold.domain.Behavior;
import manifold.domain.Glyph;
import manifold.domain.TransformEvent;
import manifold.domain.TransformListener;

/**
 * this is a class for motion of test tube, for example grasping and releasing
 * @author Chao Han
 *
 */
public class TestTubeMotionWatcher implements Behavior, TransformListener {
	private SpectroBehavior spectro;
	
	/**
	 * adds target
	 */
	public void addTarget(String objName, Object obj) {
    
		if (objName.equals("spectroBehavior")) {
			spectro = ((SpectroBehavior)obj);
			System.out.println("SpectroBehavior Target " +
					"added to TestTubeMotionwatcher...");
		} else {
			System.err.println("TestTubeMotionWatcher.addTarget: " +
					"Unknown target " + objName);
		}
	}

	public void setSelectOccured(TransformEvent event) {
	}
	
	public void graspOccured(TransformEvent event) {
    
		Glyph testTube = event.source_;

		((TestTubeFigure)testTube).putPropertyValue(
				"testTube.isWithinRange", new Boolean(false));

		if (spectro.getTestTube() == testTube) {
      
			spectro.removeTestTube();
		}
	}

	public void transformOccured(TransformEvent event) {
	}

	/**
	 * action of releasing
	 * if test tube is put in the right range, it drops into sample holder automatically
	 * if test tube is not put in the right range, it move back to its original position 
	 * automatically
	 */
	public void releaseOccured(TransformEvent event) {
	
		Glyph testTube = event.source_;
		Transform2D trans = (Transform2D)testTube.getTransform();

		
		if ((Math.abs(trans.x_ - 72.0D) <= 10.0D) &&                          /*range of position than test tube can be*/
				(Math.abs(trans.y_ - 389.0D) <= 60.0D) &&                     /*dropped into sample holder. width is 10, height is 60*/
				 	(spectro.seeIfTestTubeCanBePlaced())) {                  /*if test tube can be inserted*/
      
			((TestTubeFigure)testTube).putPropertyValue(
					"testTube.isWithinRange", new Boolean(true));
			Toolkit.getDefaultToolkit().beep();
			Transform2D testTubePosition = new Transform2D(71.0D,             /*destination position for inserting a test tube*/
					450.0D, 0.0D);
			event.source_.transform(testTubePosition);
			spectro.setTestTube((TestTubeFigure)testTube);
		} else {                                                              /*if test tube is not positioned in the right range*/
			if ((Math.abs(trans.x_ - 72.0D) <= 20.0D) &&                      /*another range of position*/
					(Math.abs(trans.y_ - 389.0D) <= 90.0D) && 
						(spectro.seeIfTestTubeCanBePlaced())) {
				try {
					spectro.testTubeHelpNecessary();                         /*inspire help*/
				} catch (JessException e) {
					System.out.println("Test tube exception " + e);
				}
			}
			
			/*move to the original position automatically*/
			double originalTestTubePosX = ((Double)((TestTubeFigure)testTube).
				getPropertyValue("glyph.initialX")).doubleValue();
			double originalTestTubePosY = ((Double)((TestTubeFigure)testTube).
				getPropertyValue("glyph.initialY")).doubleValue();
			Transform2D originalPos = new Transform2D(originalTestTubePosX,
					originalTestTubePosY, 0.0D);
			testTube.transform(originalPos);
		}
	}
}

