package biology.spectro.presentation;

import biology.presentation.tools.ClickManipulator;
import biology.spectro.domain.AbsorbanceMeter;
import flatscape.presentation.FlatscapeDocumentView;
import flatscape.presentation.GlyphViewTraversal2D;
import flatscape.presentation.figures.ImageView;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import manifold.domain.Glyph;
import manifold.presentation.Manipulator;
/**
 * This class draw Absorbance meter.
 * @author Chao Han
 */

public class AbsorbanceMeterView extends ImageView {
	
	/**
	 * Constructor for AbsorbanceMeterView
	 * @param glyph   
	 * @param docView 
	 */
	public AbsorbanceMeterView(Glyph glyph, FlatscapeDocumentView docView){
		super(glyph, docView);
	}

	/**
	 * determine whether the meter on device is clicked, if clicked, display 
	 * enlarged view of meter
	 * @param event Event of mouse
	 */	
	public Manipulator createManipulator(MouseEvent event){
		String isclicked = (String)model_.getPropertyValue("glyph.permittedUserTransform");
		
		/*if AbsorbanceMeter is clicked*/
		if (isclicked.equals("click")) {
			return new ClickManipulator(documentView_, this);
		}
		
		new Exception("PowerSwitchView.createManipulator: Unknown Transform").printStackTrace();
		return null;
	}
	
	/**
	 * Draws meter on device and the large view of it.
	 */
	protected void draw(GlyphViewTraversal2D glyph) {
		Graphics2D meterView = glyph.getPainter();
		
		if (fillColor_ != null){
			fillColor_ = meterView.getColor();
			meterView.setColor(fillColor_);
		}
		
		Object obj = NumberFormat.getInstance();
		((DecimalFormat)obj).setMinimumFractionDigits(0);
		((DecimalFormat)obj).setMaximumFractionDigits(2);
		((DecimalFormat)obj).setMinimumIntegerDigits(0);
		
		double scaleX = 0.3D;
		double scaleY = 0.3D;
		meterView.scale(scaleX, scaleY);  
		
		meterView.translate(-90, 230);                                  /*set position of meter on the device*/
		
		double initialAngle = -2.617993877991494D;                      /*-150degree,210degree*/
		double intervalAngle = 0.2094395102393195D;                     /*interval angle = 12 degree*/
		
		int radius = 150;
		meterView.drawArc(-50, -260, 300, 300, 37, 106);                /*draw curve on meter*/
		
		/* Draws scales on device*/
		for (int j = 0; j < 11; j++) {
			double posX = radius * Math.cos(initialAngle) + 100.0D;
			double posY = radius * Math.sin(initialAngle) - 130.0D;
			
			if (initialAngle < 0.5235987755982988D) {                   /*30 degree*/ 
				meterView.drawString(new Integer(j * 10).toString(), 
						(int)(posX + 10.0D * Math.cos(initialAngle)),
						(int)(posY + 10.0D * Math.sin(initialAngle)));
				meterView.drawLine((int)posX, (int)posY, 
						(int)(posX - 10.0D * Math.cos(initialAngle)),
						(int)(posY - 10.0D * Math.sin(initialAngle)));
			}
			initialAngle += intervalAngle;                              /*12 degree*/
		}
		BasicStroke scaleThickness = new BasicStroke(0.05F);
		meterView.setStroke(scaleThickness);
		meterView.setColor(Color.black);
		
		boolean bool = ((Boolean)((AbsorbanceMeter)model_).
				getPropertyValue("absorbanceMeter.needEnlargedView")).booleanValue();
		if (bool == true){                                      /*large view needs to display*/
			meterView.translate(60, -220);                      /*sets position of Large view*/
			meterView.scale(1.0D / scaleX, 1.0D / scaleY);      /*sets the size of Large view*/
			
			
			meterView.setColor(new Color(255, 255, 255, 255));  /*sets background color of Large view*/
			meterView.fillRect(-75, -305, 400, 200);            /*sets position of background of Large view */
			meterView.setColor(Color.black);	                /*sets scale color of Large View*/
			
			
			initialAngle = -2.617993877991494D;                 /*-150degree, 210degree*/
			
			meterView.drawArc(-50, -260, 300, 300, 37, 106);    /*draw transmittance curve*/  
			
			radius = 150;   
			
			/*draw transmittance scale on large view*/
			for (int k = 0; k < 11; k++){
				double posXLargeTrans = radius * Math.cos(initialAngle) + 100.0D;
				double posYLargeTrans = radius * Math.sin(initialAngle) - 130.0D;
				if (initialAngle < 0.5235987755982988D){
					meterView.drawString(new Integer(k * 10).toString(), 
							(int)(posXLargeTrans + 10.0D * Math.cos(initialAngle)), 
							(int)(posYLargeTrans + 10.0D * Math.sin(initialAngle)));
					meterView.drawLine((int)posXLargeTrans, (int)posYLargeTrans, 
							(int)(posXLargeTrans - 10.0D * Math.cos(initialAngle)),
							(int)(posYLargeTrans - 10.0D * Math.sin(initialAngle)));
				}
				initialAngle += intervalAngle;
			}
			
			/*draw absorbance scale on large view*/
			initialAngle = -0.698131700797732D;                      /*40degree*/
			intervalAngle = 0.174532925199433D;                      /*10degree*/
			meterView.drawArc(-30, -249, 260, 300, 40, 100);         /*draw absorbance curve*/
			radius = 125;
			
			double transValue = 1.0D;                                /*transmittance scale /100*/
			for (int m = 10; m >= 0; m--){
				double posXLargeAbsorbance = radius * Math.cos(initialAngle) + 102.0D;
				double posYLargeAbsorbance = radius * Math.sin(initialAngle) - 110.0D;
				if (m != 0) {
					meterView.drawString(((NumberFormat)obj).format(Math.log(1.0D / transValue) / Math.log(10.0D)),
							(int)(posXLargeAbsorbance - 20.0D * Math.cos(initialAngle)),
							(int)(posYLargeAbsorbance - 20.0D * Math.sin(initialAngle)));
				} else {
					meterView.drawString(((NumberFormat)obj).format(-Math.log(0.0D)),           /*infinite*/ 
							(int)(posXLargeAbsorbance - 20.0D * Math.cos(initialAngle)), 
							(int)(posYLargeAbsorbance - 20.0D * Math.sin(initialAngle)));
				}
				meterView.drawLine((int)posXLargeAbsorbance, (int)posYLargeAbsorbance, 
						(int)(posXLargeAbsorbance + 10.0D * Math.cos(initialAngle)), 
						(int)(posYLargeAbsorbance + 10.0D * Math.sin(initialAngle)));
				
				initialAngle -= intervalAngle;
				transValue -= 0.1D;
			}
		}
		if (stroke_ != null){
			meterView.setStroke(stroke_);
		}
	}
}