package biology.spectro.presentation;

import flatscape.presentation.FlatscapeDocumentView;
import flatscape.presentation.GlyphViewTraversal2D;
import flatscape.presentation.figures.TextView;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import manifold.domain.Glyph; 

/**
 * This class, which extends TextView, draw instruction text.
 * @author Chao Han
 *
 */
public class InstructionTextView extends TextView{
	private Color   color_;                              /*color of instruction*/                                    
	private char[]  textArray = new char[15000];         /*instructions*/
	
	/**
	 * Sets rough picture of components
	 * @param glyph     shape of a component
	 * @param docView   descriptions in XML file
	 */
	public InstructionTextView(Glyph glyph, FlatscapeDocumentView docView) {
		super(glyph, docView);
		setProperties();
	}
	
	/**
	 * This method is set properties of instruction text such as style and size etc.
	 */
	protected void setProperties(){
		text_ = ((String)model_.getPropertyValue("text"));
		int fontStyle = ((Integer)model_.getPropertyValue("font.style")).intValue();   /*font style*/
		int fontSize = ((Integer)model_.getPropertyValue("font.size")).intValue();     /*font size*/
		String fontName = (String)model_.getPropertyValue("font.name");                /*font name*/
		color_ = ((Color)model_.getPropertyValue("draw.color"));                       /*sets instructions' color*/
		fontStyle_ = new Font(fontName, fontStyle, fontSize);                          /*new font*/
	}

	public void draw(GlyphViewTraversal2D viewTraversal2D) {
		Graphics2D instructionText = viewTraversal2D.getPainter();
		instructionText.setFont(fontStyle_);                                           /*sets font style*/
		instructionText.transform(documentView_.getInverseScale());
		
		int width = 0;
		int height = 0;
		
		Rectangle2D instructionTextBoard = instructionText.getFont().
				getMaxCharBounds(instructionText.getFontRenderContext());
		
		width = new Float(-instructionTextBoard.getWidth() / 2.0D).intValue();
		height = new Float(instructionTextBoard.getHeight() / 2.0D).intValue();
		
		int textLength = text_.length();
		
		for (int m = 0; m < textLength; m++) {
			textArray[m] = text_.charAt(m);
		}
		
		for (int n = textLength; n < 15000; n++){ 
			textArray[n] = ' ';
		}
		
		instructionText.setColor(color_);                                              /*sets instructions color*/
		int lineLength = 100;                                                                  
		int lineNo = textLength / lineLength;
		
		for (int i = 0; i < lineNo; i++){
			if (textArray[(lineLength - 1 + i * lineLength)] != ' ') {
				for (int j = textLength; j > lineLength - 1 + i * lineLength; j--) {
					textArray[j] = textArray[(j - 1)];
				}
			    if (textArray[(lineLength - 2 + i * lineLength)] != ' '){
			    	textArray[(lineLength - 1 + i * lineLength)] = '-';
			    } else {
			    	textArray[(lineLength - 1 + i * lineLength)] = ' ';
			    }
			    textLength += 1;
			}
		}
		
		for (int k = 0; k < lineNo+1; k++) {
			instructionText.drawChars(textArray, k * lineLength, 
					lineLength, width - 250, height + 18 * k);
		}
		instructionText.drawChars(textArray, lineLength * lineNo,
				textLength - lineNo * lineLength, width - 250, height + 18 * lineNo);
	}
}

