package biology.spectro.presentation;

import biology.domain.NextButton;
import biology.presentation.tools.ClickManipulator;
import flatscape.presentation.FlatscapeDocumentView;
import flatscape.presentation.GlyphViewTraversal2D;
import flatscape.presentation.figures.ImageView;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.awt.geom.AffineTransform;
import manifold.domain.Glyph;
import manifold.presentation.Manipulator;

/**
 * This class draws next button
 * @author Chao Han
 *
 */
public class NextButtonView extends ImageView {
	
	/**
	 * sets next button
	 * @param glyph     rough shape
	 * @param docView   descriptions in XML file
	 */
	public NextButtonView(Glyph glyph, FlatscapeDocumentView docView) {
		super(glyph, docView);
		}
	
	/**
	 * listener for mouse click
	 * @param mouseEvent mouse Listener
	 */
	public Manipulator createManipulator(MouseEvent mouseEvent) {
		String buttonIsClicked = (String)model_.getPropertyValue("glyph.permittedUserTransform");
		if (buttonIsClicked.equals("click")){
			return new ClickManipulator(documentView_, this);
			}
		new Exception("NextButtonView.createManipulator: Unknown Transform").printStackTrace();
		return null;
	}
	
    /**
     * Draws next button
     */
	protected void draw(GlyphViewTraversal2D glyph) {
		Graphics2D nextButton = glyph.getPainter();
		if (fillColor_ != null) {
			fillColor_ = nextButton.getColor();
			nextButton.setColor(fillColor_);
		}
		
		/*sets size and position of next button*/
		nextButton.translate(-width_ / 2.0D, -height_ / 2.0D);
		Object obj = documentView_.getInverseTranslation();
		
		AffineTransform affineTrans = new AffineTransform();
		affineTrans.setToScale(((AffineTransform)obj).getScaleX(),
				((AffineTransform)obj).getScaleY());
		
		nextButton.drawImage(image_, affineTrans, null);
		
		/*sets different pictures for next button*/
		boolean buttonIsActivated = ((Boolean)model_.getPropertyValue("button.isActivated")).booleanValue();
		if (buttonIsActivated){                                                  
			((NextButton)model_).putPropertyValue("image.location", 
					"biology/spectro/resources/nextActive.gif");
		} else{
			((NextButton)model_).putPropertyValue("image.location",
					"biology/spectro/resources/nextInActive.gif");
		}  
	}
}
