package biology.spectro.presentation;

import biology.presentation.BioGUIBuilder;
import biology.presentation.tools.BioTool;
import biology.spectro.domain.SpectroMachine;
import flatscape.domain.FlatscapeDocumentFactory;
import flatscape.presentation.FlatscapeDocumentView;
import flatscape.util.Debug;
import flatscape.xml.DocumentViewProperties;
import flatscape.xml.OpenXMLDocument2D;
import java.awt.Component;
import java.util.Hashtable;
import javax.swing.JRootPane;
import manifold.domain.AppearanceEvent;
import manifold.domain.AppearanceListener;
import manifold.domain.Document;
import manifold.domain.DocumentFolder;
import manifold.domain.Glyph;
import manifold.domain.commands.CloseDocumentCommand;
import manifold.domain.commands.CreateDocumentCommand;
import manifold.presentation.DocumentView;
import manifold.presentation.Editor;
import manifold.presentation.PresentationControl;
import org.xml.sax.XMLReader;

public class SpectroEditor extends Editor implements AppearanceListener {
	protected Debug debug = new Debug(true);
	
	private static SpectroEditor editor = null;
	private static String warning;
	
	public SpectroMachine specMachine;
	
	protected FlatscapeDocumentView activeDocumentView;
	protected DocumentFolder documentFolder;
	protected PresentationControl presentationControl;
	protected BioGUIBuilder guiBuilder;
	protected JRootPane editorPanel;
	
	//Getter
	public DocumentView getActiveDocumentView() {
		return this.activeDocumentView;
	}
	public static AppearanceListener getAppearanceListener() {
		return editor;
	}
    public PresentationControl getPresentationControl() {
    	return presentationControl;
    }
    public Component getView() {
    return editorPanel;
    }
	public String getWarning() {
		return warning;
	}

	//Setter
    public void setActiveDocumentView(DocumentView paramDocumentView) {
    	activeDocumentView = ((FlatscapeDocumentView)paramDocumentView);
    }
    public void setVisibleOccured(AppearanceEvent paramAppearanceEvent) {
    
    }

	private SpectroEditor(PresentationControl paramPresentationControl) {
		presentationControl = paramPresentationControl;
		
		guiBuilder = new SpectroGUIBuilder(this);
		guiBuilder.buildView();
		editorPanel = guiBuilder.getRootPane();
	}
	
	public static SpectroEditor createEditor(PresentationControl paramPresentationControl) {
		editor = new SpectroEditor(paramPresentationControl);
		return editor;
	}
	
	public void closeDocument() {
		if (this.activeDocumentView != null) {
			Glyph document = activeDocumentView.getModel();
			debug.println("closeDocument: " + document.getId());
			CloseDocumentCommand cmd = new CloseDocumentCommand(document.getId());
			presentationControl.fireCommand(cmd);
		}
	}
	
	public void createNewDocument() {
		System.out.println("new document created");
		if (activeDocumentView == null) {
			FlatscapeDocumentFactory docFact = new FlatscapeDocumentFactory();
			CreateDocumentCommand cmd = new CreateDocumentCommand(docFact.getClass().getName(),
					"SpectroPhotometer", SpectroEditor.class.getName());
			presentationControl.fireCommand(cmd);
		}
	}
	
	public void createOccured(AppearanceEvent paramAppearanceEvent){
		Document model = (Document)paramAppearanceEvent.data_;
		//setup hardcoded properties for the new document 
		Hashtable localHashtable = getProperties();
		//create a DocumentView ( Document's View)
		activeDocumentView = new FlatscapeDocumentView(model, presentationControl, localHashtable);
		
		//guiBuilder.addJScrollPane(model.getName(), activeDocumentView.getJScrollPane());
		/*Change name of tab*/
		guiBuilder.addComponent("lab Group 5 Demo#2 Chao Han", activeDocumentView.getDrawingComponent());
		
		presentationControl.addDocumentViewToLUT(activeDocumentView);
		model.addPropertyValueChangeListener(activeDocumentView);
		
		activeDocumentView.setCurrentTool(new BioTool(this));
		presentationControl.addDocumentViewToLUT(this.activeDocumentView);
		
		model.addPropertyValueChangeListener(activeDocumentView);
		
		try {
			String parserName = "org.apache.xerces.parsers.SAXParser";
			XMLReader parser = (XMLReader)Class.forName(parserName).newInstance();
			
			Long docId = activeDocumentView.getId();
			//Display docId
			System.out.println(docId);
			
			OpenXMLDocument2D oxg = new OpenXMLDocument2D("src/biology/spectro/resources/spectrolab.xml", 
					presentationControl, parser);
			oxg.importDocument(docId);
		}
		catch (Exception e){
			System.out.println("SpectroEditor: " + e);
		}
	}
	
	public void deleteOccured(AppearanceEvent paramAppearanceEvent) { 
		debug.println("deleteOccured");
		if (activeDocumentView != null){
			//dettach the DocumentView from the Document model
			Glyph document = this.activeDocumentView.getModel();
			document.removeAppearanceListener(activeDocumentView);
			
			//remove it from the PresnetationControl's look-up table
			presentationControl.removeTarget(this.activeDocumentView.getId());
			
			// destroy the DocumentView
			activeDocumentView = null;
		} 
	}
	
	private Hashtable getProperties() {
		Hashtable properties = null;
		try{
			String parserName = "org.apache.xerces.parsers.SAXParser";
			
			XMLReader parser = (XMLReader)Class.forName(parserName).newInstance();
			DocumentViewProperties dvp = new DocumentViewProperties("src/biology/spectro/resources/BioDocumentView.xml", parser);
			properties = dvp.getProperties();
		}
		catch (Exception e) {
			System.out.println("OpenXMLHandler: " + e);
		}
		return properties;
	}
}
