package biology.spectro.presentation;

import biology.presentation.BioView;
import biology.spectro.domain.TestTubeFigure;
import flatscape.presentation.FlatscapeDocumentView;
import flatscape.presentation.GlyphViewTraversal2D;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import manifold.domain.Glyph;

/**
 * This class sets, positions and draw Test tubes
 * @author Chao Han
 *
 */
public class TestTubeView extends BioView {
	protected double width  = 20.0D;             /*width of test tube*/ 
	protected double height = 90.0D;             /*height of test tube*/
	private Color    testTubeColor;              /*color of test tube*/
	//private TestTubeFigure tt;

	/**
	 * sets test tube
	 * @param glyph     shape
	 * @param docView   descripiton in XML file
	 */
	public TestTubeView(Glyph glyph, FlatscapeDocumentView docView){
		super(glyph, docView);
		shape_ = new Rectangle2D.Double(-width / 2.0D, -height / 2.0D, width, height);
	}
	
	/**
	 * Draws a test tube
	 */
	protected void draw(GlyphViewTraversal2D view) {
		Graphics2D testTube = view.getPainter();
		Boolean testTubeIsWithinRange = (Boolean)((TestTubeFigure)model_).
				getPropertyValue("testTube.isWithinRange");
		boolean bool = testTubeIsWithinRange.booleanValue();
		if (!bool) {
			((TestTubeFigure)model_).putPropertyValue("glyph.visible", new Boolean(true));
			
			/*get color of a test tube*/
			testTubeColor = ((Color)model_.getPropertyValue("fill.color"));
			
			if ((testTubeColor.getRed() == 255) && (testTubeColor.getGreen() == 255) && 
					(testTubeColor.getBlue() == 255)){
				testTube.setColor(new Color(testTubeColor.getRed(), 
						testTubeColor.getGreen(), testTubeColor.getBlue(), 25));
			} else {
				testTube.setColor(new Color(testTubeColor.getRed(), 
						testTubeColor.getGreen(), testTubeColor.getBlue(), 255));
			}
			
			/*fill color in a test tube*/
			testTube.fillArc((int)-width / 2, (int)height * 3 / 8,            
					(int)width, (int)width, 180, 180);
			testTube.fillRect((int)-width / 2, (int)-height * 3 / 16, 
					(int)width, (int)height * 31/45);
			testTube.fillArc((int)-width / 2, (int)-height / 4, 
					(int)width, (int)width / 2, 180, 360);
 			testTube.setColor(Color.black);                                   /*sets color for the frame of a test tube*/
			
 			/*draw frame of a test tube*/
			testTube.drawArc((int)-width / 2, (int)-height / 4, (int)width,
					(int)width / 2, 180, 180);
			testTube.drawLine((int)-width / 2, (int)-height / 2, (int)-width / 2, 
					(int)height / 2);
			testTube.drawLine((int)width / 2, (int)-height / 2, (int)width / 2, 
					(int)height / 2);
			testTube.drawArc((int)-width / 2, (int)height * 3 / 8, (int)width, 
					(int)width, 180, 180);
			testTube.drawArc((int)-width / 2, (int)-height * 9 / 16, (int)width, 
					(int)width / 2,180, 360);
		} else {
			testTube.setColor((Color)model_.getPropertyValue("fill.color"));
			testTube.fillRect((int)-width / 2, 0, (int)width, 4);
		} 
		if (stroke_ != null){
			testTube.setStroke(stroke_);
		}
	}
}