package biology.spectro.presentation;

import flatscape.presentation.FlatscapeDocumentView;
import flatscape.presentation.GlyphViewTraversal2D;
import flatscape.presentation.figures.RectangleView;
import java.awt.Color;
import java.awt.Graphics2D;
import manifold.domain.Glyph;

/**
 * sets and draw wavelength meter
 * @author Chao Han
 *
 */
public class WaveMeterReadingView extends RectangleView {
	protected double width_ = 50.0D;
	protected double height_ = 50.0D;
	
	/**
	 * sets Wavelength meter
	 * @param glyph
	 * @param view
	 */
	public WaveMeterReadingView(Glyph glyph, FlatscapeDocumentView docView){
		super(glyph, docView);
	} 
	
	/**
	 * draws wavelength meter
	 */
	protected void draw(GlyphViewTraversal2D viewTraversal2D){
		Graphics2D waveMeter = viewTraversal2D.getPainter();
		if (fillColor_ != null) {
			Color color = waveMeter.getColor();
			waveMeter.setColor(color);                                            /*sets color of the meter's frame*/
		}
		
		/*draws frame of meter*/
		waveMeter.drawLine((int)-width_ / 2, 0, (int)width_ / 2, 0);
		waveMeter.drawLine((int)-width_ / 2, 0, (int)-width_ * 2 / 3,
				(int)-height_ / 2);
		waveMeter.drawLine((int)width_ / 2, 0, (int)width_ * 2 / 3,
				(int)-height_ / 2);
		waveMeter.setColor(Color.white);                                          /*sets color of scale on wave meter*/    
		
		/*draw scale*/
		double initialAngle = -0.3141592653589793D;
		int i = 30;
		for (int j = 0; j < 9; j++) {
			double posX = i * Math.cos(initialAngle);
			double posY = i * Math.sin(initialAngle) - 5.0D;
			waveMeter.drawLine((int)posX, (int)posY, 
					(int)(posX - 5.0D * Math.cos(initialAngle)),
					(int)(posY - 5.0D * Math.sin(initialAngle)));
			initialAngle -= 0.3141592653589793D;                                  /*-180 degree*/
		}
		
		waveMeter.setColor(new Color(255, 255, 255, 125));                        /*sets color of meter*/
		
		/*fill color of meter*/
		waveMeter.fillArc((int)-width_ * 2 / 3, (int)-height_ / 2 - 10,           
				(int)(width_ * 11.0D / 8.0D), 20, 0, 180);
		int[] posXs = { (int)-width_ / 2, (int)-width_ * 2 / 3, 
				(int)width_ * 2 / 3, (int)width_ / 2 };
		int[] posYs = { 0, (int)-height_ / 2, (int)-height_ / 2, 0 };
		waveMeter.fillPolygon(posXs, posYs, 4);                                  
	}
}
