package biology.spectro.domain;

import biology.domain.LabObject;
import flatscape.domain.Transform2D;

import java.awt.geom.Rectangle2D;
import java.util.HashSet;
import java.util.Iterator;
import manifold.domain.CreateGlyphCommand;
import manifold.domain.GlyphProperty;

public class SampleHolder extends LabObject {

	/**
	 * Constructor, add properties.
	 * @param cmd
	 * @param transform
	 */
	public SampleHolder(CreateGlyphCommand cmd, Transform2D transform){
		super(cmd, transform);
		addGlyphProperty("sampleHolder.isOccupied", 
				new GlyphProperty(new Boolean(false), 
						"tells if sample holder is occupied", false));
		addGlyphProperty("sampleHolder.isOpen", 
				new GlyphProperty(new Boolean(false), 
						"tells if the lid is open", true));
		addGlyphProperty("glyph.toolTip", 
				new GlyphProperty(new String("Sample Holder"),
						"Tool tip for the glyph", true));
	}
	/**
	 * Changes position of lid when click it. 
	 */
	public void grasp() {
		boolean isSampleHolderOpen = 
				((Boolean)getPropertyValue("sampleHolder.isOpen")).booleanValue();
		putPropertyValue("sampleHolder.isOpen", new Boolean(!isSampleHolderOpen));
		Transform2D lidTransform = (Transform2D)transform_;
		
		/*if sample holder is not open, -90 degree; if open 0 degree*/
		if (!isSampleHolderOpen){
			lidTransform.setRotation(-90.0D);
		} else {
			lidTransform.setRotation(0.0D);
		}
		transform(lidTransform);
	}

	/**
	 * puts property values into sample holder
	 */
	public synchronized void putPropertyValue(String propertyName, Object obj) {
		super.putPropertyValue(propertyName, obj);
		double widthOrHeight;
		Rectangle2D.Float lid = new Rectangle2D.Float();
		
		if (propertyName.equals("glyph.width")) {

			widthOrHeight = ((Double)obj).doubleValue();
			lid = (Rectangle2D.Float)getPropertyValue("glyph2D.bbox");
			lid.setRect(lid.getX(), lid.getY(), widthOrHeight, lid.getHeight());
			putPropertyValue("glyph2D.bbox", lid);
		}
		
		if (propertyName.equals("glyph.height")){
			widthOrHeight = ((Double)obj).doubleValue();
			lid = (Rectangle2D.Float)getPropertyValue("glyph2D.bbox");
			lid.setRect(lid.getX(), lid.getY(), lid.getWidth(), widthOrHeight);
			putPropertyValue("glyph2D.bbox", lid);
		}
	}
	
	/**
	 * sets test tube 
	 * @param testTube
	 */
	public void setTestTube(TestTube testTube) {
		fireSetTestTubeEvent();
	}

	/**
	 * remove test tube from sample holder
	 */
	public void removeTestTube() {
    
		fireRemoveTestTubeEvent();
	}

	/**
	 * fires flip lid event
	 */
	protected void fireFlipLidEvent() {
	  
		HashSet localHashSet = (HashSet)stateListeners_.clone();
    
		for (Iterator iter = localHashSet.iterator(); iter.hasNext(); )
			((StateListener)iter.next()).flipLid();
	}

	/**
	 * fires set test tube event
	 */
	protected void fireSetTestTubeEvent() {
	
		HashSet localHashSet = (HashSet)stateListeners_.clone();
    
		for (Iterator localIterator = localHashSet.iterator();
				localIterator.hasNext(); )
			((StateListener)localIterator.next()).setTestTube();
  
	}

	/**
	 * fire remove test tube event
	 */
	protected void fireRemoveTestTubeEvent() {
		
		HashSet localHashSet = (HashSet)stateListeners_.clone();
		
		for (Iterator localIterator = localHashSet.iterator(); localIterator.hasNext(); )
			((StateListener)localIterator.next()).removeTestTube();
	}
}